float4x4 tW : WORLD;
Texture2D texWorld <string uiname="World";>;
float4x4 tExtCam <string uiname="transform ExtCam";>;
Texture2D texExtCam <string uiname="texture External cam";>;
Texture2D texDetect <string uiname="texture Detection";>;
int maxDots;
bool colorize;
float2 Resolution;

SamplerState sPoint : IMMUTABLE
{
    Filter = MIN_MAG_MIP_POINT;
    AddressU = Border;
    AddressV = Border;
};

#include "../fxh/_PointData.fxh"
#include "../fxh/Helper.fxh"
AppendStructuredBuffer<pointData> pcBuffer : BACKBUFFER;

//==============================================================================
//COMPUTE SHADER ===============================================================
//==============================================================================

[numthreads(8, 8, 1)]
void CSBuildPointcloudBuffer( uint3 i : SV_DispatchThreadID )
{
	uint w,h, dummy;
	texWorld.GetDimensions(0,w,h,dummy);
	if (i.x >= asuint(Resolution.x) || i.y >= asuint(Resolution.y)) { return; }
	float2 coord = i.xy * float2(w / Resolution.x, h / Resolution.y) / float2(w,h);
	float4 world = texWorld.SampleLevel(sPoint, coord,0 );
	if (world.z > 0.1f){

		float4 pos = world;
		pos = mul(pos, tW);
			
		float4 PosW;
		PosW.xyz = pos.xyz;
		PosW.w = 1.0f;
		
		float4 TexCd = mul(PosW, tExtCam);
		TexCd /= TexCd.w;
		TexCd.y *= -1.0f;
		TexCd.xy += 1.0f;
		TexCd.xy /= 2.0f;
		
		float4 colPt = float4(0,0,0,1);
		int group = 0;
		if(colorize){
			colPt = texExtCam.SampleLevel(sPoint, TexCd.xy,0 );
		}

		float4 colDetect = texDetect.SampleLevel(sPoint, TexCd.xy,0 );
		if (colDetect.r>0)
		{
			colPt = colDetect;
			group = round(colDetect.r*maxDots);
		}
		
		int colPtI = encodeColor(colPt);		
		pointData pd = {pos.xyz, colPtI, group};
		pcBuffer.Append(pd);
	}
}

//==============================================================================
//TECHNIQUES ===================================================================
//==============================================================================

technique11 BuildPointcloudBuffer
{
	pass P0
	{
		SetComputeShader( CompileShader( cs_5_0, CSBuildPointcloudBuffer() ) );
	}
}
